
import sys                  # access to command-line args
import string               # string functions
import traceback
import re
from omniORB import CORBA   # omniORBpy used to generate Python client stubs
import OPUS                 # client stubs
import opusutil             # utilites like PrintMsg

# exit codes
NOT_OK = 1
OK = 0

# exceptions
class InteractiveMode(Exception):
  pass

class MissingIORonCommandLine(Exception):
  pass

class NarrowOfOrbObjectFailed(Exception):
  pass

class FailedToRegisterPipelineApp(Exception):
  pass

class FailedResourceInit(Exception):
  pass

class DatasetNotFoundInOSF(Exception):
  pass

class DataIdNotFoundInOSF(Exception):
  pass

class HaltEvent(Exception):
  pass

# set up a regular expression for detecting trailing underscores
TRAILING_UNDERSCORES = re.compile("_+$")

class InternalPoller:
  """
########################################################################
Class: InternalPoller

Description:
------------
This class provides a base class for an OPUS internal polling task.

Members:
--------
._papp - the pipeline application context

Methods:
--------
.is_interactive() - determines if task is being run in interactive mode
.pipeline_setup() - handles initialization of the pipelin environment
.run() - the event loop handling
.get_resource_items() - returns a dictionary of resource file values

########################################################################
  """
  def __init__(self):
    # constructor
    self._papp = None
    self._eventTriggerName = None
    #
    # raises exception if in interactive mode
    self.is_interactive()
    #
    # mode is pipeline, so set up
    self.pipeline_setup()

  #-------------------------------------------------------------------------
  # 
  # Name: is_interactive
  #
  # Description: If the -i command line option (and a value) are present
  #              then this task is in pipeline mode, otherwise it is
  #              in interactive mode.
  #
  # Input: global sys.argv
  #
  # Exceptions: InteractiveMode raised if that is the mode
  #
  # History:
  # 01/31/03 46981 MSwam       first version
  #
  #-------------------------------------------------------------------------
  def is_interactive(self):
    #
    # Search for the -i option and a value on the command line.
    # (It has the object reference for the UserContext CORBA object.)
    # If it is NOT present, then we are in command-line (interactive) mode.
    #
    for i in range(len(sys.argv)):
      if sys.argv[i] == "-i" and i < (len(sys.argv) - 1):
        #
        # -i argument found plus at least one more argument, 
        # so we are in pipeline mode
        return
    #
    # no -i argument or no value for it, so we are in interactive mode
    raise InteractiveMode

  #-------------------------------------------------------------------------
  # 
  # Name: pipeline_setup
  #
  # Description: The generic set-up calls for an internal poller.
  #
  # Input: command line args from sys module
  #
  # History:
  # 01/31/03 46981 MSwam       first version
  # 06/30/04 50703 MSwam       rework
  #
  #-------------------------------------------------------------------------
  def pipeline_setup(self):
    #
    # grab the -i option from the command line: it has
    # the object reference for the UserContext CORBA object.
    #
    ior = ""
    for i in range(len(sys.argv)):
      if sys.argv[i] == "-i" and i < (len(sys.argv) - 1):
        ior = sys.argv[i+1]
        break
    if ior == "":
      opusutil.PrintMsg("F","unable to locate '-i ior' on command line.")
      raise MissingIORonCommandLine
  
    #
    # Initialize the CORBA orb (this is always the same)
    #
    orb = CORBA.ORB_init(sys.argv, CORBA.ORB_ID)
  
    # Convert the stringified object reference to a CORBA object
    #
    obj = orb.string_to_object(ior)
  
    # Narrow the object to its actual type (like a cast)
    #
    uc = obj._narrow(OPUS.OPUS_API.OPUSUSER.UserContext)
    if uc == None:
      opusutil.PrintMsg("E","Narrow failed")
      raise NarrowOfOrbObjectFailed
  
    # Use the UserContext object to "register" this application with OPUS.
    # Must do for internal pollers; xpoll does this for external pollers.
    #
    try:
      self._papp = uc.registerPipelineApp(sys.argv)
    except:
      traceback.print_exc()
      opusutil.PrintMsg("E","Failed to register application")
      raise FailedToRegisterPipelineApp

  #-------------------------------------------------------------------------
  # 
  # Name: get_resource_items
  #
  # Description: Retrieves items by name from the resource file.
  #
  # Input: 
  #   keynames - a list of resource file keyword names
  #
  # Output:
  #   keyvalues - a dictionary of the keyname,keyvalue pairs
  #
  # Exceptions:
  #   FailedResourceInit - failed to find a particular resource entry
  #   
  # History:
  # 01/31/03 46981 MSwam       first version
  #
  #-------------------------------------------------------------------------
  def get_resource_items(self, keynames):
    #
    # get values from pipeline resource file
    resource_items = {}
    for keyname in keynames:
      try:
        resource_items[keyname] = self._papp.getResourceValue(keyname)
      except:
        traceback.print_exc()
        opusutil.PrintMsg("E",
                         "Failed to get resource entry: "+keyname)
        raise FailedResourceInit
    #
    return resource_items

  #-------------------------------------------------------------------------
  # 
  # Name: run
  #
  # Description: Provides the event-handling loop 
  #
  # Input: 
  #   none
  #
  # Output:
  #   none
  #
  # Exceptions:
  #   
  #   
  # History:
  # 06/30/04 50703 MSwam       first version (thanks Chris Sontag)
  # 08/06/04 49860 Sontag      Changed an IDL field name
  # 12/17/04 51433 MSwam       Use proper IGNEVT name
  #
  #-------------------------------------------------------------------------
  def run(self):
    #
    opusutil.PrintMsg("I","Entering event loop...")
    while 1:
      # make sure output from the last event goes out asap
      sys.stdout.flush()
      #
      # poll for next event
      try:
        event = self._papp.poll()
        self._eventTriggerName = event.eventName
      except:
        traceback.print_exc()
        opusutil.PrintMsg("E", "Polling failed.")
        raise

      # handle event types
      if event.theEventType == OPUS.OPUS_API.OPUSUSER.HALT:
        self.processHaltEvent(event)
      elif event.theEventType == OPUS.OPUS_API.OPUSUSER.FILE:
        self.processFileEvent(event)
      elif event.theEventType == OPUS.OPUS_API.OPUSUSER.OSF:
        self.processOsfEvent(event)
      elif event.theEventType == OPUS.OPUS_API.OPUSUSER.TIME:
        self.processTimeEvent(event)
      else: # add reinit and command later, but IGNORE for now
        opusutil.PrintMsg("E","Ignoring unhandled event type")
        try:
          self._papp.closeEvent("IGNEVT", event.eid)
        except:
          traceback.print_exc()
          opusutil.PrintMsg("E","Failed to close IGNEVT ")
    #
    # never gets here

  #-------------------------------------------------------------------------
  # 
  # Name: processHaltEvent
  #
  # Description: Default HALT event handler 
  #              (application will override if needed)
  # Input: 
  #   event - not currently used (passed for consistency with other handlers)
  #
  # Output:
  #   none
  #
  # Exceptions:
  #   
  #   
  # History:
  # 06/30/04 50703 MSwam       first version (thanks Chris Sontag)
  #
  #-------------------------------------------------------------------------
  def processHaltEvent(self, event):
    opusutil.PrintMsg("I","Received HALT event. Stopping.")
    raise HaltEvent

  #-------------------------------------------------------------------------
  # 
  # Name: processFileEvent
  #
  # Description: Default FILE event handler 
  #              (application will override if needed)
  # Input: 
  #   event - event being handled
  #
  # Output:
  #   none
  #
  # Exceptions:
  #   
  #   
  # History:
  # 06/30/04 50703 MSwam       first version (thanks Chris Sontag)
  # 12/17/04 51433 MSwam       Use proper IGNEVT name
  #
  #-------------------------------------------------------------------------
  def processFileEvent(self, event):
    opusutil.PrintMsg("W","Received FILE event. Application didn't "+
                          "care, so the base handler will close as IGNORE.")
    try:
      self._papp.closeEvent("IGNEVT", event.eid)
    except:
      traceback.print_exc()
      opusutil.PrintMsg("E","Failed to close IGNEVT ")

  #-------------------------------------------------------------------------
  # 
  # Name: processOsfEvent
  #
  # Description: Default OSF event handler 
  #              (application will override if needed)
  # Input: 
  #   event - event being handled
  #
  # Output:
  #   none
  #
  # Exceptions:
  #   
  #   
  # History:
  # 06/30/04 50703 MSwam       first version (thanks Chris Sontag)
  # 12/17/04 51433 MSwam       Use proper IGNEVT name
  #
  #-------------------------------------------------------------------------
  def processOsfEvent(self, event):
    opusutil.PrintMsg("W","Received OSF event. Application didn't "+
                          "care, so the base handler will close as IGNORE.")
    try:
      self._papp.closeEvent("IGNEVT", event.eid)
    except:
      traceback.print_exc()
      opusutil.PrintMsg("E","Failed to close IGNEVT ")

  #-------------------------------------------------------------------------
  # 
  # Name: processTimeEvent
  #
  # Description: Default TIME event handler 
  #              (application will override if needed)
  # Input: 
  #   event - event being handled
  #
  # Output:
  #   none
  #
  # Exceptions:
  #   
  #   
  # History:
  # 06/30/04 50703 MSwam       first version (thanks Chris Sontag)
  # 12/17/04 51433 MSwam       Use proper IGNEVT name
  #
  #-------------------------------------------------------------------------
  def processTimeEvent(self, event):
    opusutil.PrintMsg("W","Received TIME event. Application didn't "+
                          "care, so the base handler will close as IGNORE.")
    try:
      self._papp.closeEvent("IGNEVT", event.eid)
    except:
      traceback.print_exc()
      opusutil.PrintMsg("E","Failed to close IGNEVT ")

  #-------------------------------------------------------------------------
  # 
  # Name: getDatasetDataidFromOsfEvent
  #
  # Description: Returns the dataset and data_id values from an OSF event
  #
  # Input: 
  #   event - event being handled
  #
  # Output:
  #   dataset
  #   data_id
  #
  # Exceptions:
  #   DatasetNotFoundInOSF
  #   DataidNotFoundInOSF 
  #   
  # History:
  # 06/30/04 50703 MSwam       first version (thanks Chris Sontag)
  # 06/12/07 57369 MSwam       add trailing underscore trim for data_id
  #-------------------------------------------------------------------------
  def getDatasetDataidFromOsfEvent(self, event):
    #
    # get config entry w/Field data
    bbData = event.bb.getState()
    datasetIDENT = -999   # a "junk" bad number
    dataidIDENT = -999
    for field in bbData.entryInfo:
      if field.value == "DATASET":
        datasetIDENT = field.id
      elif field.value == "DATA_ID":
        dataidIDENT = field.id
    #
    # check if datasetIDENT was not filled
    if datasetIDENT == -999:
      opusutil.PrintMsg("E","Failed to find DATASET in triggering OSF")
      raise DatasetNotFoundInOSF
    # check if dataidIDENT was not filled
    if dataidIDENT == -999:
      opusutil.PrintMsg("E","Failed to find DATA_ID in triggering OSF")
      raise DataIdNotFoundInOSF

    # Get dataset name from event trigger
    dataset = "unknown"
    data_id = "xxx"
    for field in event.triggers[0]:
      if field.id == datasetIDENT:
        # get field.value, use strip to rm trailing whitespace and
        # a compiled regex to rm trailing underscores
        dataset = re.sub(TRAILING_UNDERSCORES,"",field.value.strip())
      elif field.id == dataidIDENT:
        data_id = re.sub(TRAILING_UNDERSCORES,"",field.value.strip())
    #
    return dataset, data_id

  #-------------------------------------------------------------------------
  # 
  # Name: getDatasetFromOsfEvent
  #
  # Description: Returns the dataset value from a FILE event
  #
  # Input: 
  #   event - event being handled
  #
  # Output:
  #   dataset
  #
  # Exceptions:
  #   DatasetNotFoundInOSF 
  #   
  # History:
  # 03/06/08 59220 Heller      add get name from file event 
  #-------------------------------------------------------------------------

  def getDatasetFromFileEvent(self, event):
    #
    # get config entry w/Field data
    bbData = event.bb.getState()
    rootnameID = -999   # a "junk" bad number
    for field in bbData.entryInfo:
      if field.value == "ROOTNAME":
        rootnameID = field.id
    #
    # check if datasetIDENT was not filled
    if rootnameID == -999:
      opusutil.PrintMsg("E","Failed to find ROOTNAME in triggering FILE")
      raise DatasetNotFoundInOSF

    # Get dataset name from event trigger
    dataset = "unknown"
    for field in event.triggers[0]:
      if field.id == rootnameID:
        dataset = field.value.strip()

    return dataset 
